import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.3
import TableModel 0.1

TableView {
    id: availablePrintersView
    width: 110
    height: 160
    anchors.fill: parent
    columnSpacing: 0
    rowSpacing: 0
    clip: true
    model: available_printers_model

    topMargin: columnsHeader.implicitHeight
    property var columnwidths: available_printers_model.getColumnWidths();
    columnWidthProvider: function (column) {
        return columnwidths[column];
        //return available_printers_model.getColumnWidth(column)
    }
    rowHeightProvider: function(row) {return 35;}
    reuseItems: true


    //On load: Load printer list from smb
    Component.onCompleted: {
        //Try to get a local printer definition list
        if (available_printers_model.loadLocalPrinterDefinitions() === false) {
            //If not available, load from remote
            if (available_printers_model.needCredentialsForPrinterList()) {
                smbDialog.open();
            } else {
                available_printers_model.readRemotePrinterListAnonymous();
            }
        }
    }

    //Header row
    Row {
        id: columnsHeader
        y: availablePrintersView.contentY
        z: 2
        Repeater {
            id: headerRepeater
            model: availablePrintersView.columns > 0 ? availablePrintersView.columns : 1
            Label {
                width: availablePrintersView.columnWidthProvider(modelData) + 1
                height: rowHeightProvider(0);
                text: available_printers_model.headerData(modelData, Qt.Horizontal)
                font.pixelSize: 15
                color: styleUtil.getFontColor()
                padding: 10
                elide: Text.ElideRight
                verticalAlignment: Text.AlignVCenter
                background: Rectangle { color: styleUtil.getTableHeaderColor() }
                //Bottom header separator
                Rectangle {
                    width: parent.width
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.margins: 0
                    height: 2
                    color: styleUtil.getTableSeparatorColor()
                }
                //Drag handle
                Rectangle {
                    id: dragRectangle
                    anchors.right: parent.right
                    anchors.top: parent.top
                    anchors.margins: 0
                    width: 2
                    height: availablePrintersView.contentHeight - availablePrintersView.contentY
                    color: styleUtil.getTableSeparatorColor()
                    MouseArea {
                        id: dragArea
                        property real dragStart: 0
                        anchors.fill: parent
                        cursorShape: Qt.SizeHorCursor

                        drag.target: parent
                        drag.axis: Drag.XAxis

                        hoverEnabled: true
                        onMouseXChanged: {
                            if(drag.active){
                                var label = headerRepeater.itemAt(index);
                                var labelWidth = label.width + mouseX;
                                var columnWidth = columnwidths[index] + mouseX;
                                if(label.width < 50 || columnWidth < 50) {
                                    label.width = 50
                                    columnwidths[index] = 50;
                                } else {
                                    label.width = labelWidth;
                                    columnwidths[index] = columnWidth;
                                }
                                availablePrintersView.forceLayout();
                            }
                        }

                        //If drag finished: Store column widths
                        drag.onActiveChanged: function () {
                            if (!drag.active) {
                                available_printers_model.storeColumnWidth(index, columnWidthProvider(index));
                            }
                        }
                    }
                }
            }
        }

    }

    //Printer info popup
    Dialog {
        id: printerInfoPopup
        x: 100
        y: 100
        width: 450
        height: 375
        modal: false
        focus: true
        closePolicy: Popup.CloseOnEscape | Popup.CloseOnPressOutsideParent
        standardButtons: Dialog.Ok

        Component.onCompleted: {
            standardButton(Dialog.Ok).text = qsTr("Ok")
        }

        contentItem: ScrollView {
            clip: true
            anchors.left: parent.left
            anchors.leftMargin: 25
            anchors.top: parent.top
            anchors.topMargin: 25
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 50
            width: 300
            height: 450

            PrinterInfo {
                id: printerInfo
                width: parent.width
                height: parent.height
            }
        }
    }

    //Context menu after right click on printer
    Menu {
        id: printerContextMenu
        MenuItem {
            text: qsTr("Install")
            onTriggered: {
                printerInstallDialog.open()
            }
        }
        MenuItem {
            text: qsTr("Info")
            onTriggered: {
                printerInfoPopup.open()
                printerInfo.initData()
            }
        }
    }

    delegate: Rectangle {
        color: available_printers_model.isCellSelected(row, column) === true ?
                   styleUtil.getAccentColor(): styleUtil.getBackgroundColor();
        Text {
            text: display
            anchors.fill: parent
            anchors.margins: 10
            color: styleUtil.getFontColor()
            font.pixelSize: 15
            verticalAlignment: Text.AlignVCenter
            elide: Text.ElideRight
            ToolTip.visible: cellMouseArea.containsMouse && truncated
            ToolTip.text: text
        }

        MouseArea {
            id: cellMouseArea
            anchors.fill: parent
            acceptedButtons: Qt.LeftButton | Qt.RightButton
            hoverEnabled: true
            onClicked: {
                available_printers_model.handleItemClicked(row, column)
                if (mouse.button === Qt.RightButton) {
                    printerContextMenu.popup()
                }
            }
        }
    }


    Rectangle {
        z: 3
        color: styleUtil.getTableHeaderColor()
        y: availablePrintersView.contentY
        x: availablePrintersView.contentX
        width: availablePrintersView.leftMargin
        height: availablePrintersView.topMargin
    }
}
