import QtQuick 2.2
import QtQuick.Controls 2.2
import QtQuick.Dialogs 1.0 as Dialogs

Dialog {
    height: 400
    width: 600
    standardButtons: Dialog.Apply | Dialog.Cancel

    modal: false
    onApplied: {
        installResultText.text = available_printers_model.addSelectedPrinter(driverFileText.text);
        installResultDialog.open()
    }

    BusyIndicator {
        id: installDialogBusyIndicator
        anchors.centerIn: parent
        running: true
    }

    Component.onCompleted: {
        standardButton(Dialog.Apply).text = qsTr("Apply")
        standardButton(Dialog.Cancel).text = qsTr("Cancel")
        reset()
    }

    contentItem: Pane {
        Label {
            id: installDialogLabel
            anchors.top: parent.top
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
        }
        Label {
            id: installCmdTextLabel
            anchors.top: installDialogLabel.bottom
            anchors.topMargin: 15
            anchors.left: parent.left
            anchors.leftMargin: 5
            font.bold: true
            text: qsTr("Command")
        }

        TextField {
            id: installCmdText
            width: 400
            readOnly: true
            selectByMouse: true
            anchors.top: installCmdTextLabel.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5

        }
        Button {
            id: installCmdCopyButton
            text: qsTr("Copy")
            anchors.top: installCmdTextLabel.bottom
            anchors.topMargin: 5
            anchors.left: installCmdText.right
            anchors.leftMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            flat: true
            onClicked: {
                //Copy the cmd to clipboard using the invisible TextEdit
                installDialogClipboard.selectAll();
                installDialogClipboard.copy();
            }
        }
        Label {
            id: driverFileLabel
            anchors.top: installCmdText.bottom
            anchors.topMargin: 15
            anchors.left: parent.left
            anchors.leftMargin: 5
            text: qsTr("Driver")
            font.bold: true
        }
        TextField {
            id: driverFileText
            width: 400
            readOnly: true
            selectByMouse: true
            anchors.top: driverFileLabel.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5

        }
        Button {
            id: searchDriverFileButton
            text: qsTr("Search...")
            anchors.top: driverFileLabel.bottom
            anchors.topMargin: 5
            anchors.left: driverFileText.right
            anchors.leftMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            flat: true
            onClicked: {
                fileDialog.open()
            }
        }
    }

    //Invisble TextEdit used to copy the cmd to clipboard
    TextEdit {
        id: installDialogClipboard
        text: installCmdText.text
        visible: false
    }

    function reset() {
        installDialogBusyIndicator.running = true;
        installDialogLabel.text = "";
        installCmdText.text = "";
        driverFileText.text = "";
    }

    onOpened: {
        var queue = available_printers_model.getSelectedQueueName();
        installDialogLabel.text = qsTr("Installing printer %1").arg(queue)
        var cmd = available_printers_model.getSelectedInstallCmd();
        installCmdText.text = cmd;
        var driver = available_printers_model.getSelectedDriver();
        driverFileText.placeholderText = driver;
        installDialogBusyIndicator.running = false;
    }

    Dialogs.FileDialog {
        id: fileDialog
        height: 450
        width: 600
        title: qsTr("Please choose a ppd file")
        folder: shortcuts.home
        selectExisting: true
        selectMultiple: false
        selectFolder: false
        nameFilters: [ qsTr("PPD files (*.ppd)")]

        onAccepted: {
            var file = fileDialog.fileUrl.toString();
            file = file.substring(7);
            driverFileText.text = file;
            installCmdText.text = available_printers_model.getSelectedInstallCmd(file);
        }
    }

    onClosed: reset();
}
