import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Dialogs 1.1
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12

Dialog {
    x: 100
    y: 100
    width: 425
    height: 300

    header: Text {
        anchors.top: parent.top
        anchors.topMargin: 40
        anchors.left: parent.left
        anchors.leftMargin: 20
        text: qsTr("Enter SMB credentials to access printer list on %1").arg(controller.getSmbUri(false))
        font.pixelSize: 12
    }

    standardButtons: Dialog.Ok | Dialog.Cancel

    Component.onCompleted: {
        standardButton(Dialog.Ok).text = qsTr("Ok")
        standardButton(Dialog.Cancel).text = qsTr("Cancel")
    }

    onAccepted: {
        submit();
    }

    onRejected: setLoading(false);

    function submit() {
        try {
            available_printers_model.readRemotePrinterListAuthenticated(
                    smbUser.text.toString(), smbPassword.text.toString(), smbDomain.text);
            smbUser.clear();
            smbPassword.clear();
            close();
        } catch (e) {
            if (e.message.includes("NT_STATUS_ACCESS_DENIED")
                    || e.message.includes("NT_STATUS_LOGON_FAILURE")) {
                smbErrorText.text =  qsTr("Please check credentials and try again")
            } else if (e.message.includes("NT_STATUS_UNSUCCESSFUL")) {
                smbErrorText.text = qsTr("The connection to the host could not be established.\nPlease check connection and domain settings and try again.")
            } else if (e.message.includes("NT_STATUS_OBJECT_NAME_NOT_FOUND")) {
                smbErrorText.text = qsTr("The printer list could not be found on the remote location.\nPlease check settings and try again.")
            } else if (e.message.includes("NT_STATUS_BAD_NETWORK_NAME")) {
                smbErrorText.text = qsTr("Could not access the SMB Share.\nCheck if you have sufficient permission to access the given directory.")
            } else {
                smbErrorText.text = "";
            }

            smbErrorDialog.open();
        }

        setLoading(false);
    }

    contentItem: GridLayout {
        anchors.top: parent.top
        anchors.topMargin: 75
        anchors.left: parent.left
        anchors.leftMargin: 20        
        anchors.right: parent.right
        anchors.rightMargin: 20
        anchors.bottom: parent.Bottom
        anchors.bottomMargin: 75
        columns: 2

        Label {
            id: usernameLabel
            text: qsTr("Username:")
            font.bold: true
        }
        TextField {
            id: smbUser
            placeholderText: qsTr("Enter name")
            focus: true
        }
        Label {
            id: passwordLabel
            text: qsTr("Password:")
            font.bold: true
        }
        TextField {
            id: smbPassword
            placeholderText: qsTr("Enter password")
            echoMode: TextInput.Password
            Keys.onReturnPressed: {
                submit();
            }
        }
        Label {
            id: domainLabel
            text: qsTr("Domain:")
            font.bold: true
        }
        TextField {
            id: smbDomain
            placeholderText: qsTr("Domain")
            Keys.onReturnPressed: {
                submit();
            }
        }
    }

    Dialog {
        id: smbErrorDialog
        x:printerInstallDialog.x + 25
        y:printerInstallDialog.y + 25
        height: 150
        width: 450
        modal: false
        standardButtons: Dialog.Ok
        contentItem: Pane {
            Text {
                id: smbErrorHeader
                text: qsTr("Error reading the remote printer list")
            }
            Text {
                id: smbErrorText
                anchors.top: smbErrorHeader.bottom
                anchors.margins: 5
                text: ""
            }
        }
    }
}
