import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Controls.Material 2.12
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12

Window {
    id: mainWindow
    visible: true
    width: 640
    height: 480
    title: "CupsConfigApp"
    color: styleUtil.getMainWindowBackground()

    Material.theme: Material.Light
    Material.accent: Material.LightBlue

    Component.onCompleted: {
        styleUtil.switchTheme(controller.getConfigOption(controller.CFG_NAME_UI_THEME));
    }

    function setLoading (loading) {
        reloadBusyIndicator.running = loading;
    }

    //About Dialog
    About {
        id: aboutDialog
    }

    //SMB credentials popup
    SMBDialog {
        id: smbDialog
    }

    //Settings Dialog
    Settings {
        id: settingsDialog
    }

    MenuBar{
        id: toolBar
        height: 40
        anchors.right: parent.right
        anchors.rightMargin: 0
        anchors.left: parent.left
        anchors.leftMargin: 0
        anchors.top: parent.top
        anchors.topMargin: 0

        Menu {
            title: qsTr("&File")
            Action {
                text: qsTr("&Settings")
                onTriggered: settingsDialog.open()
            }
            Action {
                text: qsTr("&Quit")
                onTriggered: Qt.quit()
            }
        }
        Menu {
            title: qsTr("&Help")
            Action {
                text: qsTr("&About")
                onTriggered: aboutDialog.open()
            }
        }
    }

    Pane {
        id: printerScrollViewContainer
        Material.elevation: 6
        anchors.right: parent.right
        anchors.rightMargin: 10
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 10
        anchors.top: availPrintersTools.bottom
        anchors.topMargin: 10
        ScrollView {
            id: scrollView
            anchors.right: parent.right
            anchors.rightMargin: 0
            anchors.left: parent.left
            anchors.leftMargin: 10
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 0
            anchors.top: parent.top
            anchors.topMargin: 0

            ScrollBar.vertical.policy: ScrollBar.AsNeeded
            ScrollBar.horizontal.policy: ScrollBar.AsNeeded

            AvailablePrinters {
                id: availablePrinters
            }
        }
    }


    Pane {
        id: availPrintersTools
        height: 35
        anchors.right: parent.right
        anchors.rightMargin: 10
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.top: toolBar.bottom
        anchors.topMargin: 10

        Material.elevation: 6

        Label {
            id: availablePrintersTitle
            height: 50
            text: qsTr("Available network printers")
            verticalAlignment: Text.AlignVCenter
            anchors.left: parent.left
            anchors.leftMargin: 0
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 0
            anchors.top: parent.top
            anchors.topMargin: 0
            font.pixelSize: 16
        }

        BusyIndicator {
            id: reloadBusyIndicator
            running: false
            anchors.right: reloadButton.left
            anchors.rightMargin: 10
            anchors.top: parent.top
            anchors.topMargin: -14
            width: 35
            height: 35
        }

        Button {
            id: reloadButton
            x: 5
            y: 0
            width: 175
            height: 35
            text: qsTr("Reload printers")
            anchors.right: installButton.left
            anchors.rightMargin: 10
            anchors.top: parent.top
            anchors.topMargin: -14
            onClicked: {
                setLoading(true);
                if (available_printers_model.needCredentialsForPrinterList()) {
                    smbDialog.open();
                } else {
                    available_printers_model.readRemotePrinterListAnonymous();
                    setLoading(false);
                }
            }
        }

        Button {
            id: installButton
            x: 5
            y: 0
            width: 75
            height: 35
            text: qsTr("Install")
            anchors.right: printerSearchField.left
            anchors.rightMargin: 10
            anchors.top: parent.top
            anchors.topMargin: -14
            onClicked: {
                printerInstallDialog.open()
            }
        }

        // Printer search components
        TextField {
            id: printerSearchField
            width: 200
            height: 25
            placeholderText: qsTr("Search printers")
            anchors.right: parent.right
            anchors.rightMargin: 0
            anchors.top: parent.top
            anchors.topMargin: -14
            anchors.bottom: parent.bottom
            anchors.bottomMargin: -19
            font.pointSize: 10
            onAccepted: {
                //filter();
            }
            function filter() {
                setLoading(true);
                if (printerSearchField.text.length > 0) {
                    available_printers_model.filter(printerSearchField.text)
                } else {
                    available_printers_model.clearFilter();
                }
                setLoading(false);
            }

            //Timer triggering the filtering after finishing
            Timer {
                id: printerSearchTimer
                interval: 750
                repeat: false
                triggeredOnStart: false
                onTriggered: {
                    printerSearchField.filter();
                }
            }

            //Button to clear the search field
            RoundButton {
                id: printerSearchFieldClearButton
                width: 25
                height: 25
                radius: 12.5
                text: "x"
                font.pointSize: 10
                flat: true

                anchors.right: parent.right
                anchors.rightMargin: 0
                anchors.top: parent.top
                anchors.topMargin: 5

                onClicked: {
                    printerSearchField.clear()
                    //printerSearchField.filter();
                }
            }
        }
    }

    PrinterInstallDialog {
        id: printerInstallDialog
        x: 100
        y: 100
    }

    Dialog {
        id: installResultDialog
        x:printerInstallDialog.x + 25
        y:printerInstallDialog.y + 25
        height: 150
        width: 250
        modal: false
        standardButtons: Dialog.Ok

        Component.onCompleted: {
            standardButton(Dialog.Ok).text = qsTr("Ok")
        }

        contentItem: Text {
            id: installResultText
            anchors.fill: parent
            anchors.margins: 5
            text: ""
        }
    }

    //Utility functions
    StyleUtil {
        id: styleUtil
    }
}

/*##^##
Designer {
    D{i:1;anchors_width:360;anchors_x:0;anchors_y:0}D{i:5;anchors_x:181;anchors_y:46}
D{i:4;anchors_height:200;anchors_width:200;anchors_x:250;anchors_y:125}D{i:7;anchors_x:111;anchors_y:46}
D{i:8;anchors_y:0}D{i:6;anchors_height:200;anchors_width:200}
}
##^##*/
