#ifndef CONFIGUTILITY_H
#define CONFIGUTILITY_H

#include <map>
#include <regex>
#include <string>

class ConfigUtility {

public:
    static ConfigUtility* getInstance();

    void reloadCfg();

    /**
     * @brief Get the value of a cfg option by the given key.
     * @throws ConfigException if key was not found
     * @param key Cfg key
     * @return Option as string
     */
    std::string getCfgOpt(std::string key);

    /**
     * @brief Set the value of a cfg option by the given key.
     * @Throws ConfigException if key was not found
     * @param key Option key
     * @param value New value
     */
    void setCfgOPt(std::string key, std::string value);

    void saveUserCfg();

    //Constants
    //Config keys
    const std::string CFG_NAME_SMB_HOST = "smbhost";
    const std::string CFG_NAME_SMB_SHARE = "smbshare";
    const std::string CFG_NAME_SMB_FILE = "smbfile";
    const std::string CFG_NAME_UI_LANG = "language";
    const std::string CFG_NAME_UI_THEME = "theme";
    const std::string CFG_NAME_UI_COLUMN_WIDTHS = "column_widths";

    //Config values
    const std::string CFG_VALUE_THEME_LIGHT = "light";
    const std::string CFG_VALUE_THEME_DARK = "dark";

    const std::string CFG_VALUE_LANG_DE_CH = "de_CH";
    const std::string CFG_VALUE_LANG_EN_US = "en_US";
    const std::string CFG_VALUE_LANG_AUTO = "auto";

private:
    ConfigUtility();

    //User config
    std::string cfgFilePath;
    std::string fullPath;
    const std::string cfgFilePathPostfix = ".config/cups-config-app";
    const std::string cfgFileName = "config";

    //Global config
    const std::string globalCfgPath = "/etc/cups-config-app";
    const std::string globalCfgFileName = "config";
    std::string globalFullPath;

    static ConfigUtility* instance;
    std::map<std::string, std::string> *options;

    bool checkGlobalCfgFile();
    bool checkUserCfgFile();
    void createCfgDir();
    void initCfg();
    void readCfgFile();
    void readCfgFile(std::string path);

    const std::regex CFG_FILE_COMMENT_REGEX = std::regex("(!|#)");
    const std::string CFG_FILE_DELIMITER = "=";

};

class ConfigException: public std::exception {
private:
    std::string message;
public:
    explicit ConfigException(const std::string& message) {
        this->message = message;
    };
    const char* what() const noexcept override {
        return message.c_str();
    }
};

class IOException: public ConfigException {
public:
    explicit IOException(const std::string& message): ConfigException(message) {};
};

class ConfigParserException: public ConfigException {
public:
    explicit ConfigParserException(const std::string& message): ConfigException(message) {};
};

#endif // CONFIGUTILITY_H
