#ifndef CONTROLLER_H
#define CONTROLLER_H

#include "configutility.h"
#include "smbutility.h"

#include <QObject>

class Controller : public QObject {
    Q_OBJECT
public:
    explicit Controller(QObject *parent = nullptr);

    Q_INVOKABLE void reloadCfg();
    Q_INVOKABLE bool saveConfig();
    Q_INVOKABLE bool setConfigOption(QString key, QString value);
    Q_INVOKABLE QString getConfigOption(QString key);

    /**
     * @brief Get the build time as QString
     * @return Build time as QString
     */
    Q_INVOKABLE QString getBuildTime();

    /**
     * @brief Get the hg commit hash as QString
     * @return Commit hash as QString
     */
    Q_INVOKABLE QString getCommitHash();

    /**
     * @brief Get the smb uri using the current settings
     * @param includeFile Set if filename should be included
     * @return Uri as QString
     */
    Q_INVOKABLE QString getSmbUri(bool includeFile);

    /**
     * @brief Get the smb uri using the given values
     * @param host Host
     * @param share Share
     * @param QString file
     * @return Uri as QString
     */
    Q_INVOKABLE QString getSmbUri(QString host, QString share, QString file);

    /**
     * @brief Get application version as QString
     *
     * Format: MAJOR.MINOR.PATCH
     * @return Version as QString
     */
    Q_INVOKABLE QString getVersion();

    //Expose constants to QML
    //Config names
    Q_PROPERTY(QString CFG_NAME_SMB_HOST READ getCfgNameSmbHost)
    Q_PROPERTY(QString CFG_NAME_SMB_SHARE READ getCfgNameSmbShare)    
    Q_PROPERTY(QString CFG_NAME_SMB_FILE READ getCfgNameSmbFile)    
    Q_PROPERTY(QString CFG_NAME_UI_LANG READ getCfgNameUiLang)
    Q_PROPERTY(QString CFG_NAME_UI_THEME READ getCfgNameUiTheme)

    //Config values
    Q_PROPERTY(QString CFG_VALUE_THEME_LIGHT READ getCfgValueThemeLight CONSTANT);
    Q_PROPERTY(QString CFG_VALUE_THEME_DARK READ getCfgValueThemeDark CONSTANT);
    Q_PROPERTY(QString CFG_VALUE_LANG_AUTO READ getCfgValueLangAuto CONSTANT);

private:
    ConfigUtility* cfg;    
    SMBUtility* smbUtility;

    QString getCfgNameSmbHost();
    QString getCfgNameSmbShare();    
    QString getCfgNameSmbFile();
    QString getCfgNameUiLang();
    QString getCfgNameUiTheme();

    QString getCfgValueThemeLight();
    QString getCfgValueThemeDark();
    QString getCfgValueLangAuto();

signals:

};

#endif // CONTROLLER_H
