#include <QCommandLineOption>
#include <QCommandLineParser>

#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QQuickStyle>

#include <QtDebug>
#include <iostream>

#include "controller.h"
#include "smbutility.h"
#include "tablemodel.h"
#include "translator.h"

int main(int argc, char *argv[]) {

    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);

    QGuiApplication app(argc, argv);

    app.setApplicationName("CupsConfigApp");
    app.setOrganizationName("Some Company");
    app.setOrganizationDomain("somecompany.com");

    //Parse cmd options
    QCommandLineParser parser;
    QCommandLineOption debugOpt("debug", "Debug level", "debug", "0");
    parser.addOption(debugOpt);
    parser.process(app);
    int debugValue = parser.value(debugOpt).toInt();
    SMBUtility::setDebugLevel(debugValue);

    QQuickStyle::setStyle("Material");

    QQmlApplicationEngine engine;
    const QUrl url(QStringLiteral("qrc:/main.qml"));

    //Create translator
    Translator translator;
    ConfigUtility *cfg = ConfigUtility::getInstance();
    translator.setEngine(&engine);
    translator.switchLanguage(cfg->getCfgOpt(cfg->CFG_NAME_UI_LANG));

    engine.rootContext()->setContextProperty("translator", &translator);

    qmlRegisterType<TableModel>("TableModel", 0, 1, "TableModel");

    TableModel available_printers_model;
    engine.rootContext()->setContextProperty("available_printers_model", &available_printers_model);

    Controller controller;
    engine.rootContext()->setContextProperty("controller", &controller);

    qInfo() << "Version: " << APPLICATION_VERSION;
    qInfo() << "Commit: " << HG_COMMIT;
    qInfo() << "Built at: " << BUILD_TIME;

    QObject::connect(&engine, &QQmlApplicationEngine::objectCreated,
                     &app, [url](QObject *obj, const QUrl &objUrl) {
        if (!obj && url == objUrl)
            QCoreApplication::exit(-1);
    }, Qt::QueuedConnection);
    engine.load(url);
    return app.exec();
}
