#include "printermodel.h"

PrinterModel::PrinterModel(std::string name, std::string server, std::string location,
                           std::string driver, std::string format, std::string color,
                           std::string duplex, std::string staple, std::string punch,
                           std::string booklet, std::string scanner, std::string opts,
                           std::string comment) {
    this->name = name;
    this->server = server;
    this->location = location;
    this->driver = driver;
    this->format = format;
    this->color = color.compare("color");
    this->duplex = duplex.compare("duplex");
    this->staple = staple.compare("staple");
    this->punch = punch.compare("punch");
    this->booklet = booklet.compare("bookelet");
    this->scanner = scanner;
    this->opts = opts;
    this->comment = comment;
}


std::string PrinterModel::getAtIndexAsStr(int index) {
    switch (index) {
    case 0: return getName();
    case 1: return getServer();
    case 2: return getLocation();
    case 3: return getDriver();
    case 4: return getFormat();
    case 5: return getColor()?  "color": "grayscale";
    case 6: return getDuplex()? "duplex": "simplex";
    case 7: return getStaple()? "staple": "nostaple";
    case 8: return getPunch()? "punch": "nopunch";
    case 9: return getBooklet()? "booklet": "nobooklet";
    case 10: return getScanner();
    case 11: return getOpts();
    case 12: return getComment();
    default: return "n/a";
    }
}

bool PrinterModel::matches(const std::regex regex) {
    return std::regex_search(name, regex)
            || std::regex_search(server, regex)
            || std::regex_search(location, regex)
            || std::regex_search(driver, regex)
            || std::regex_search(comment, regex);
}

std::string PrinterModel::getName() {
    return this->name;
}

std::string PrinterModel::getServer() {
    return this->server;
}

std::string PrinterModel::getLocation() {
    return this->location;
}

std::string PrinterModel::getDriver() {
    return this->driver;
}

std::string PrinterModel::getFormat() {
    return this->format;
}

bool PrinterModel::getColor() {
    return this->color;
}

bool PrinterModel::getDuplex() {
    return this->duplex;
}

bool PrinterModel::getStaple() {
    return this->staple;
}

bool PrinterModel::getPunch() {
    return this->punch;
}

bool PrinterModel::getBooklet() {
    return this->booklet;
}

std::string PrinterModel::getScanner() {
    return this->scanner;
}

std::string PrinterModel::getOpts() {
    return this->opts;
}

std::string PrinterModel::getComment() {
    return this->comment;
}
