#include "printerparser.h"

#include <codecvt>
#include <fstream>
#include <iostream>
#include <locale>
#include <sstream>
#include <string>

#include <QtDebug>

using namespace std;
PrinterParser::PrinterParser(){ }

vector<PrinterModel>* PrinterParser::getPrinters(std::string file) {
    std::vector<PrinterModel> *printers = new std::vector<PrinterModel>();
    ifstream printersFile(file, ios::binary);

    if (printersFile.is_open()){
        std::stringstream sStream;
        //Skip bom
        printersFile.seekg(2);
        //Buffer in stringstream
        sStream << printersFile.rdbuf();
        std::string bytes = sStream.str();
        //Ensure proper length
        int len = bytes.size();
        if(len % 2) {
            len--;
        }
        //Create utf16 string and convert to utf8
        std::wstring sw;
        for(size_t i = 0; i < len;)
        {
            //little-endian
            int lo = bytes[i++] & 0xFF;
            int hi = bytes[i++] & 0xFF;
            sw.push_back(hi << 8 | lo);
        }

        wstring_convert<std::codecvt_utf8_utf16<wchar_t>> convert;
        stringstream utf8 (convert.to_bytes(sw));
        //Parse lines
        string line;
        while(getline(utf8, line)){
            //Skip empty lines
            if (line.size() == 0) {
                continue;
            }
            if (line.find("#") != 0) {
                printers->push_back(createPrinterModel(line));
            }
        }
        printersFile.close();
    } else {
        qDebug() << "Could not open file" << QString::fromStdString(file);
    }
    return printers;
}

PrinterModel PrinterParser::createPrinterModel(std::string printerString) {
    vector<string> parts;
    size_t pos = 0;
    string token;
    string delimiter = "\t";
    while ((pos = printerString.find(delimiter)) != string::npos) {
        token = printerString.substr(0, pos);
        parts.push_back(token);
        printerString.erase(0, pos + delimiter.length());
    }
    //Push the remaining token
    parts.push_back(printerString);
    if (parts.size() < 12) {
        std::cout << "Error parsing line starting with: "
                  << parts.begin()->c_str()
                  << " ..., number of elements is " << parts.size()
                  << ", but expecting 13." << std::endl;
        throw new PrinterParserException();
    }
    if (parts.size() == 12) {
        parts.push_back("");
    }
    PrinterModel model = PrinterModel(
                parts.at(0), parts.at(1), parts.at(2), parts.at(3), parts.at(4),
                parts.at(5), parts.at(6), parts.at(7), parts.at(8), parts.at(9),
                parts.at(10), parts.at(11), parts.at(12));
    return model;
}
