﻿#include "smbutility.h"

#include <array>
#include <cstdio>
#include <iostream>
#include <memory>
#include <stdexcept>
#include <regex>
#include <sstream>
#include <string>
#include <syscall.h>

#include <QDebug>

int SMBUtility::debugLevel = 0;

SMBUtility::SMBUtility() {
    cfg = ConfigUtility::getInstance();
}



std::string SMBUtility::exec(const std::string cmd) {
    std::array<char, 128> buffer;
    std::string result;

    auto pipe = popen(cmd.c_str(), "r");

    if (!pipe){
        throw std::runtime_error("popen() failed!");
    }

    while (!feof(pipe)) {
        if (fgets(buffer.data(), 128, pipe) != nullptr)
            result += buffer.data();
    }

    auto rc = pclose(pipe);
    if (rc != EXIT_SUCCESS) {
        throw new SMBException(result);
    }
    return result;
}

std::string SMBUtility::getRemoteConfigFile(std::string user, std::string password, std::string domain) {

    std::string credentials = user + "\%" + password;
    credentials = std::regex_replace(credentials, std::regex("\""), "\\\"");


    std::stringstream cmdStream;
    cmdStream << CMD;
    if (domain.length() > 0) {
        cmdStream << CMD_DOMAIN_PREFIX
                  << domain
                  << CMD_DOMAIN_POSTFIX;
    }
    cmdStream << CMD_CREDENTIALS_PREFIX
          << credentials
          << CMD_CREDENTIALS_POSTFIX
          << getSmbURI()
          << CMD_SOURCE_PREFIX
          << cfg->getCfgOpt(cfg->CFG_NAME_SMB_FILE)
          << CMD_DEST_PREFIX
          << TMP_PRINTER_FILE
          << CMD_DEST_POSTFIX
          << CMD_OUTPUT_POSTFIX;
    std::string cmdString = cmdStream.str();
    if (debugLevel == 2) {
        qDebug() << cmdString.c_str();
    }
    std::string result = exec(cmdString);
    return TMP_PRINTER_FILE;
}

std::string SMBUtility::getRemoteConfigFileAnonymous() {
    std::stringstream cmdStream;
    cmdStream << CMD
              << CMD_ANON
              << getSmbURI()
              << CMD_SOURCE_PREFIX
              << cfg->getCfgOpt(cfg->CFG_NAME_SMB_FILE)
              << CMD_DEST_PREFIX
              << TMP_PRINTER_FILE
              << CMD_DEST_POSTFIX
              << CMD_NO_OUTPUT_POSTFIX;
    std::string cmdString = cmdStream.str();
    int result = std::system(cmdString.c_str());
    if (result > 0) {
        throw new SMBException("");
    }
    return TMP_PRINTER_FILE;
}

std::string SMBUtility::getSmbURI() {
    return getSmbURI(false);
}

std::string SMBUtility::getSmbURI(bool includeFile) {
    std::string host = cfg->getCfgOpt(cfg->CFG_NAME_SMB_HOST);
    std::string share = cfg->getCfgOpt(cfg->CFG_NAME_SMB_SHARE);
    std::string file = cfg->getCfgOpt(cfg->CFG_NAME_SMB_FILE);
    return getSmbURI(host, share, includeFile? file: "");
}

std::string SMBUtility::getSmbURI(std::string host, std::string share, std::string file) {
    std::stringstream uri;
    if (host.substr(0, 2).compare("//") != 0) {
        uri  << "//";
    }

    uri << host;
    if (host.find_last_of("/") != host.length() - 1) {
        uri << "/";
    }
    uri << share;
    if (file.length() > 0) {
        uri << "/" << file;
    }
    return uri.str();
}

void SMBUtility::setDebugLevel(int debugLevel) {
    SMBUtility::debugLevel = debugLevel;

}
