#include "translator.h"

#include <QDebug>

#include <QCoreApplication>
#include <QLocale>
#include <QQmlEngine>
#include <stdexcept>

Translator::Translator(QObject *parent) : QObject(parent) {
    de_CH = new QTranslator();
    en_US = new QTranslator();
    autoLang = new QTranslator();
    if (de_CH->load(":/CupsConfigApp_de_CH.qm")) {
        qDebug() << "Loaded de_CH";
    } else {
        qDebug() << "de_CH failed";
    }
    if (en_US->load(":/CupsConfigApp_en_US.qm")) {
        qDebug() << "Loaded en_US";
    } else {
        qDebug() << "en_US failed";
    }

    translators.insert(std::pair<std::string, QTranslator*>("de_CH", de_CH));
    translators.insert(std::pair<std::string, QTranslator*>("en_US", en_US));
    translators.insert(std::pair<std::string, QTranslator*>("auto", autoLang));


    //Get system locale
    updateSystemLocale();

}

void Translator::setEngine(QQmlApplicationEngine *engine) {
    this->engine = engine;
}

void Translator::switchLanguage(QString locale) {
    switchLanguage(locale.toStdString());
}

void Translator::switchLanguage(std::string locale) {
    updateSystemLocale();
    qDebug() << "Switching locale to:" << locale.c_str();
    QTranslator* translator;
    try {
        translator = translators.at(locale);
    } catch (std::out_of_range) {
        translator = translators.at("en_US");

    }
    QCoreApplication::removeTranslator(current);
    QCoreApplication::installTranslator(translator);
    current = translator;
    engine->retranslate();
}

void Translator::updateSystemLocale() {
    QLocale system = QLocale::system();
    switch (system.language()) {
        case QLocale::English:
            autoLang = en_US;
            break;
    case QLocale::German:
            autoLang = de_CH;
            break;
    default:
        autoLang = en_US;
    }

}
